"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _draftJs = require("draft-js");

var _react = _interopRequireDefault(require("react"));

var _constants = require("./constants");

var _adjustBlockDepth = _interopRequireDefault(require("./modifiers/adjustBlockDepth"));

var _resetInlineStyle = _interopRequireDefault(require("./modifiers/resetInlineStyle"));

var _image = _interopRequireDefault(require("./decorators/image"));

var _link = _interopRequireDefault(require("./decorators/link"));

var _changeCurrentBlockType = _interopRequireDefault(require("./modifiers/changeCurrentBlockType"));

var _handleBlockType = _interopRequireDefault(require("./modifiers/handleBlockType"));

var _handleImage = _interopRequireDefault(require("./modifiers/handleImage"));

var _handleInlineStyle = _interopRequireDefault(require("./modifiers/handleInlineStyle"));

var _handleLink = _interopRequireDefault(require("./modifiers/handleLink"));

var _handleNewCodeBlock = _interopRequireDefault(require("./modifiers/handleNewCodeBlock"));

var _insertEmptyBlock = _interopRequireDefault(require("./modifiers/insertEmptyBlock"));

var _insertText = _interopRequireDefault(require("./modifiers/insertText"));

var _leaveList = _interopRequireDefault(require("./modifiers/leaveList"));

var _splitBlockAndChange = _interopRequireDefault(require("./modifiers/splitBlockAndChange"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultLanguages = {
  bash: "Bash",
  c: "C",
  cpp: "C++",
  css: "CSS",
  go: "Go",
  html: "HTML",
  java: "Java",
  js: "JavaScript",
  kotlin: "Kotlin",
  mathml: "MathML",
  perl: "Perl",
  ruby: "Ruby",
  scala: "Scala",
  sql: "SQL",
  svg: "SVG",
  swift: "Swift"
};
var INLINE_STYLE_CHARACTERS = ["*", "_", "`", "~"];

var defaultRenderSelect = function defaultRenderSelect(_ref) {
  var options = _ref.options,
      onChange = _ref.onChange,
      selectedValue = _ref.selectedValue;
  return _react["default"].createElement("select", {
    value: selectedValue,
    onChange: onChange
  }, options.map(function (_ref2) {
    var label = _ref2.label,
        value = _ref2.value;
    return _react["default"].createElement("option", {
      key: value,
      value: value
    }, label, "}");
  }));
};

function inLink(editorState) {
  var selection = editorState.getSelection();
  var contentState = editorState.getCurrentContent();
  var block = contentState.getBlockForKey(selection.getAnchorKey());
  var entityKey = block.getEntityAt(selection.getFocusOffset());
  return entityKey != null && contentState.getEntity(entityKey).getType() === "LINK";
}

function inCodeBlock(editorState) {
  var startKey = editorState.getSelection().getStartKey();

  if (startKey) {
    var currentBlockType = editorState.getCurrentContent().getBlockForKey(startKey).getType();
    if (currentBlockType === "code-block") return true;
  }

  return false;
}

function checkCharacterForState(config, editorState, character) {
  var newEditorState = (0, _handleBlockType["default"])(config.features.block, editorState, character);

  if (editorState === newEditorState && config.features.inline.includes("IMAGE")) {
    newEditorState = (0, _handleImage["default"])(editorState, character, config.entityType.IMAGE);
  }

  if (editorState === newEditorState && config.features.inline.includes("LINK")) {
    newEditorState = (0, _handleLink["default"])(editorState, character, config.entityType.LINK);
  }

  if (newEditorState === editorState && config.features.block.includes("CODE")) {
    var contentState = editorState.getCurrentContent();
    var selection = editorState.getSelection();
    var key = selection.getStartKey();
    var currentBlock = contentState.getBlockForKey(key);
    var text = currentBlock.getText();
    var type = currentBlock.getType();
    if (type !== "code-block" && _constants.CODE_BLOCK_REGEX.test(text)) newEditorState = (0, _handleNewCodeBlock["default"])(editorState);
  }

  if (editorState === newEditorState) {
    newEditorState = (0, _handleInlineStyle["default"])(config.features.inline, editorState, character);
  }

  return newEditorState;
}

function checkReturnForState(config, editorState, ev) {
  var newEditorState = editorState;
  var contentState = editorState.getCurrentContent();
  var selection = editorState.getSelection();
  var isCollapsed = selection.isCollapsed();
  var key = selection.getStartKey();
  var endOffset = selection.getEndOffset();
  var currentBlock = contentState.getBlockForKey(key);
  var blockLength = currentBlock.getLength();
  var type = currentBlock.getType();
  var text = currentBlock.getText();

  if (/-list-item$/.test(type) && text === "") {
    newEditorState = (0, _leaveList["default"])(editorState);
  }

  var isHeader = /^header-/.test(type);
  var isBlockQuote = type === "blockquote";
  var isAtEndOfLine = endOffset === blockLength;
  var atEndOfHeader = isHeader && isAtEndOfLine;
  var atEndOfBlockQuote = isBlockQuote && isAtEndOfLine;

  if (newEditorState === editorState && isCollapsed && (atEndOfHeader || atEndOfBlockQuote)) {
    // transform markdown (if we aren't in a codeblock that is)
    if (!inCodeBlock(editorState)) {
      newEditorState = checkCharacterForState(config, newEditorState, "\n");
    }

    if (newEditorState === editorState) {
      newEditorState = (0, _insertEmptyBlock["default"])(newEditorState);
    } else {
      newEditorState = _draftJs.RichUtils.toggleBlockType(newEditorState, type);
    }
  } else if (isCollapsed && (isHeader || isBlockQuote) && !isAtEndOfLine) {
    newEditorState = (0, _splitBlockAndChange["default"])(newEditorState);
  }

  if (newEditorState === editorState && type !== "code-block" && config.features.block.includes("CODE") && _constants.CODE_BLOCK_REGEX.test(text)) {
    newEditorState = (0, _handleNewCodeBlock["default"])(editorState);
  }

  if (newEditorState === editorState && type === "code-block") {
    if (/```\s*$/.test(text)) {
      newEditorState = (0, _changeCurrentBlockType["default"])(newEditorState, type, text.replace(/```\s*$/, ""));
      newEditorState = (0, _insertEmptyBlock["default"])(newEditorState);
    } else if (ev.shiftKey) {
      newEditorState = (0, _insertEmptyBlock["default"])(newEditorState);
    } else {
      newEditorState = (0, _insertText["default"])(editorState, "\n");
    }
  }

  return newEditorState;
}

var unstickyInlineStyles = function unstickyInlineStyles(character, editorState) {
  var selection = editorState.getSelection();
  if (!selection.isCollapsed()) return editorState;

  if (editorState.getLastChangeType() !== "md-to-inline-style") {
    return editorState;
  }

  var startOffset = selection.getStartOffset();
  var content = editorState.getCurrentContent();
  var block = content.getBlockForKey(selection.getStartKey());
  var previousBlock = content.getBlockBefore(block.getKey());
  var entity = block.getEntityAt(startOffset - 1);
  if (entity !== null) return editorState; // If we're currently in a style, but the next character doesn't have a style (or doesn't exist)
  // we insert the characters manually without the inline style to "unsticky" them

  if (!startOffset && previousBlock) {
    // If we're in the beginning of the line we have to check styles of the previous block
    var previousBlockStyle = previousBlock.getInlineStyleAt(previousBlock.getText().length - 1);
    if (previousBlockStyle.size === 0) return editorState;
  } else {
    var style = block.getInlineStyleAt(startOffset - 1);
    if (style.size === 0) return editorState;
  }

  var nextStyle = block.getInlineStyleAt(startOffset);
  if (nextStyle.size !== 0) return editorState;

  var newContent = _draftJs.Modifier.insertText(content, selection, character);

  return _draftJs.EditorState.push(editorState, newContent, "insert-characters");
};

var defaultConfig = {
  renderLanguageSelect: defaultRenderSelect,
  languages: defaultLanguages,
  features: {
    inline: _constants.defaultInlineWhitelist,
    block: _constants.defaultBlockWhitelist
  },
  entityType: _constants.ENTITY_TYPE
};

var createMarkdownPlugin = function createMarkdownPlugin() {
  var _config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var store = {};

  var config = _objectSpread({}, defaultConfig, {}, _config, {
    features: _objectSpread({}, defaultConfig.features, {}, _config.features),
    entityType: _objectSpread({}, defaultConfig.entityType, {}, _config.entityType)
  });

  return {
    store: store,
    decorators: [(0, _link["default"])({
      entityType: config.entityType.LINK
    }), (0, _image["default"])({
      entityType: config.entityType.IMAGE
    })],
    initialize: function initialize(_ref3) {
      var setEditorState = _ref3.setEditorState,
          getEditorState = _ref3.getEditorState;
      store.setEditorState = setEditorState;
      store.getEditorState = getEditorState;
    },
    blockStyleFn: function blockStyleFn(block) {
      switch (block.getType()) {
        default:
          break;
      }

      return null;
    },
    blockRendererFn: function blockRendererFn(block, _ref4) {
      var setReadOnly = _ref4.setReadOnly,
          getReadOnly = _ref4.getReadOnly,
          setEditorState = _ref4.setEditorState,
          getEditorState = _ref4.getEditorState,
          getEditorRef = _ref4.getEditorRef;

      switch (block.getType()) {
        case CODE_BLOCK_TYPE:
          {
            return {
              component: CodeBlock,
              props: {
                setEditorState: setEditorState,
                renderLanguageSelect: config.renderLanguageSelect,
                languages: config.languages,
                getReadOnly: getReadOnly,
                setReadOnly: setReadOnly,
                language: block.getData().get("language"),
                getEditorState: getEditorState
              }
            };
          }

        default:
          return null;
      }
    },
    onTab: function onTab(ev, _ref5) {
      var getEditorState = _ref5.getEditorState,
          setEditorState = _ref5.setEditorState;
      var editorState = getEditorState();
      var newEditorState = (0, _adjustBlockDepth["default"])(editorState, ev);

      if (newEditorState !== editorState) {
        setEditorState(newEditorState);
        return "handled";
      }

      return "not-handled";
    },
    handleReturn: function handleReturn(ev, editorState, _ref6) {
      var setEditorState = _ref6.setEditorState;
      if (inLink(editorState)) return "not-handled";
      var newEditorState = checkReturnForState(config, editorState, ev);
      var selection = newEditorState.getSelection(); // exit code blocks

      if (inCodeBlock(editorState) && !Object.is(editorState.getImmutable(), newEditorState.getImmutable())) {
        setEditorState(newEditorState);
        return "handled";
      }

      newEditorState = checkCharacterForState(config, newEditorState, "\n");
      var content = newEditorState.getCurrentContent(); // if there are actually no changes but the editorState has a
      // current inline style we want to split the block

      if (Object.is(editorState.getImmutable(), newEditorState.getImmutable()) && editorState.getCurrentInlineStyle().size > 0) {
        content = _draftJs.Modifier.splitBlock(content, selection);
      }

      newEditorState = (0, _resetInlineStyle["default"])(newEditorState);

      if (editorState !== newEditorState) {
        setEditorState(_draftJs.EditorState.push(newEditorState, content, "split-block"));
        return "handled";
      }

      return "not-handled";
    },
    handleBeforeInput: function handleBeforeInput(character, editorState, _ref7) {
      var setEditorState = _ref7.setEditorState;
      // If we're in a code block - don't transform markdown
      if (inCodeBlock(editorState)) return "not-handled"; // If we're in a link - don't transform markdown

      if (inLink(editorState)) return "not-handled";
      var unsticky = unstickyInlineStyles(character, editorState);

      if (editorState !== unsticky) {
        setEditorState(unsticky);
        return "handled";
      }

      var newEditorState = checkCharacterForState(config, editorState, character);

      if (editorState !== newEditorState) {
        setEditorState(newEditorState);
        return "handled";
      }

      return "not-handled";
    },
    handlePastedText: function handlePastedText(text, html, editorState, _ref8) {
      var setEditorState = _ref8.setEditorState;

      if (inCodeBlock(editorState)) {
        setEditorState((0, _insertText["default"])(editorState, text));
        return "handled";
      }

      return "not-handled";
    },
    handleKeyCommand: function handleKeyCommand(command, editorState, _ref9) {
      var setEditorState = _ref9.setEditorState;

      switch (command) {
        case "backspace":
          {
            // When a styled block is the first thing in the editor,
            // you cannot delete it. Typing backspace only deletes the content
            // but never deletes the block styling.
            // This piece of code fixes the issue by changing the block type
            // to 'unstyled' if we're on the first block of the editor and it's empty
            var selection = editorState.getSelection();
            var currentBlockKey = selection.getStartKey();
            if (!currentBlockKey) return "not-handled";
            var content = editorState.getCurrentContent();
            var currentBlock = content.getBlockForKey(currentBlockKey);
            var firstBlock = content.getFirstBlock();
            if (firstBlock !== currentBlock) return "not-handled";
            var currentBlockType = currentBlock.getType();
            var isEmpty = currentBlock.getLength() === 0;
            if (!isEmpty || currentBlockType === "unstyled") return "not-handled";
            setEditorState((0, _changeCurrentBlockType["default"])(editorState, "unstyled", ""));
            return "handled";
          }

        default:
          {
            return "not-handled";
          }
      }
    }
  };
};

var _default = createMarkdownPlugin;
exports["default"] = _default;